<?php
/**
 * INTERFACE for rollout plugins
 * 
 * @author hahn
 * 
 * Axel <axel.hahn@unibe.ch>
 * 2024-08-29  Axel php8 only; added variable types
 */
interface iRolloutplugin
{

    // ----------------------------------------------------------------------
    // VERIFY
    // ----------------------------------------------------------------------

    /**
     * Get an array with shell commands to check requirements if the plugin
     * can work
     * 
     * @return array
     */
    public function checkRequirements(): array;

    /**
     * Get an array with shell commands to check access to a deploy target
     * 
     * @return array
     */
    public function checkConnectionToTarget(): array;

    // ----------------------------------------------------------------------
    // SETTER
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // GETTER
    // ----------------------------------------------------------------------

    /**
     * Get configuration array for the project .. or more specific for a given phase
     * 
     * @param  string   $sPhase
     * @param  boolean  $bMask   Flag for public output; if true then mask your secrets
     * @return array
     */
    public function getConfig(string $sPhase = '', bool $bMask = false): array;

    /**
     * Get an array with shell commands to execute for deployment of built file
     * 
     * @param  string   $sPhase
     * @param  boolean  $bMask   Flag for public output; if true then mask your secrets
     * @return array
     */
    public function getDeployCommands(string $sPhase, bool $bMask = false): array;

    /**
     * Get name of plugin as string ... language specific
     * @return string
     */
    public function getName(): string;

    /**
     * Get description of plugin as string ... language specific
     * @return string
     */
    public function getDescription(): string;

    /**
     * Get array of data in info.js
     * @return array
     */
    public function getPluginInfos(): array;

    // ----------------------------------------------------------------------
    // RENDERER
    // ----------------------------------------------------------------------

    /**
     * form renderer: show 
     * - formvars for project OR
     * - a single message that no configuration items exists
     * 
     * @return string
     */
    public function renderFormdata4Project(): string;

    /**
     * form renderer: show configuration for a given phase
     
     * @param string $sPhase  phaese; one of preview|stage|live
     * @return string
     */
    public function renderFormdata4Phase(string $sPhase): string;

    // ----------------------------------------------------------------------
    // ACTIONS
    // ----------------------------------------------------------------------


}

