<?php
/**
 * BRAINSTORMING ONLY
 * 
 * interface for a version control system
 * @author hahn
 * 
 * Axel: <axel.hahn@unibe.ch>
 * (...)
 * 2024-08-28  Axel   php8 only; added variable types; short array syntax
 */
interface iVcs {

    // ----------------------------------------------------------------------
    // get metadata 
    // ----------------------------------------------------------------------

    /**
     * return url to vcs sources
     */
    public function getUrl(): string;
    
    /**
     * return url to view sources in webrowser to generate an infolink
     */
    public function getWebGuiUrl(): string;
    
    /**
     * return the build type, i.e. git|svn|cvs|
     */
    public function getBuildType(): string;
    
    // ----------------------------------------------------------------------
    // actions
    // ----------------------------------------------------------------------

    /**
     * cleanup unneeded files and directories in a checked out directory
     * and remove all vcs specific files and directories.
     * This method works on linux only
     * 
     * @param string $sWorkDir  path of the build directory to cleanup the vcs meta data from
     * @return bool
     */
    public function cleanupWorkdir(string $sWorkDir): bool;
    
    /**
     * get current revision and commit message from remote repository
     * @param boolean  $bRefresh  optional: refresh data; default: use cache
     * @return array
     */
    public function getRepoRevision(bool $bRefresh=false);
    
    /**
     * get current revision and log message from given directory
     * @return bool|array
     */
    public function getRevision(string $sWorkDir = ''): bool|array;

    /**
     * Get sources from vsc and check them out in given directory
     * It returns false if the workdir was not found or the url for git repo is not set
     * Otherwise it retunrs the output of the checkout commands
     * 
     * @param string $sWorkDir working dir where to check out source url
     * @return bool|string
     */
    public function getSources(string $sWorkDir): bool|string;
    

}

